/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;
import org.fortiss.tooling.kernel.introspection.IIntrospectiveKernelService;
import org.fortiss.tooling.kernel.introspection.KernelIntrospectionSystemService;

/**
 * {@link ViewPart} for the marker display.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: E58AC80BFD881369E9A5E8844E5AEA0D
 */
public class KISSViewPart extends ViewPart implements ISelectionChangedListener {

	/** The greeting text. */
	private static final String GREETING =
			"Welcome to KISS!\n\nThis is the kernel introspection system service.";
	/** GUI builder viewer instance. */
	private KISSViewerGUI gui;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		parent.setLayout(new FillLayout());
		gui = new KISSViewerGUI(parent, SWT.NONE);
		TreeViewer tv = gui.getTreeViewer();
		tv.setContentProvider(new KISSViewerContentProvider());
		tv.setLabelProvider(new KISSViewerLabelProvider());
		tv.setSorter(new ViewerSorter());
		tv.addSelectionChangedListener(this);

		gui.getDescriptionText().setText(GREETING);

		// the class is used as hidden root element, which contains the instance
		// see also content provider implementation for this setup
		tv.setInput(KernelIntrospectionSystemService.class);
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		gui.getTreeViewer().getControl().setFocus();
	}

	/** Refreshes the view. Needs to be called on UI thread. */
	public void refresh() {
		if(!gui.getTreeViewer().getControl().isDisposed()) {
			gui.getTreeViewer().refresh();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		if(gui.getTreeViewer().getControl().isDisposed()) {
			return;
		}
		if(event.getSelection().isEmpty()) {
			gui.getDescriptionText().setText(GREETING);
			return;
		}
		if(event.getSelection() instanceof IStructuredSelection) {
			IStructuredSelection sel = (IStructuredSelection)event.getSelection();
			if(sel.getFirstElement() instanceof IIntrospectiveKernelService) {
				IIntrospectiveKernelService service =
						(IIntrospectiveKernelService)sel.getFirstElement();
				gui.getDescriptionText().setText(service.getIntrospectionDescription());
				return;
			}
		}
		// fallback
		gui.getDescriptionText().setText(GREETING);
	}
}
