/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.introspection;

import static java.util.Collections.unmodifiableSet;

import java.util.HashSet;
import java.util.Set;

/**
 * Implementation of the kernel introspection system service (KISS).
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class KernelIntrospectionSystemService implements IIntrospectiveKernelService {

	/** Returns the singleton instance of the service. */
	public static final KernelIntrospectionSystemService INSTANCE =
			new KernelIntrospectionSystemService();

	/** The introspective services of the kernel. */
	private final HashSet<IIntrospectiveKernelService> services =
			new HashSet<IIntrospectiveKernelService>();

	/** Singleton constructor. */
	private KernelIntrospectionSystemService() {
		// nothing to do here
	}

	/**
	 * Registers the given {@link IIntrospectiveKernelService} service with
	 * {@link KernelIntrospectionSystemService KISS}.
	 */
	public void registerService(IIntrospectiveKernelService service) {
		services.add(service);
	}

	/** Unregisters the given service. */
	public void unregisterService(IIntrospectiveKernelService service) {
		services.remove(service);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Kernel Introspection System Service";
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Set<IIntrospectionItem> getIntrospectionItems() {
		return unmodifiableSet(services);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		return getIntrospectionLabel() +
				"\n\nThis service manages the kernel introspection system." +
				"\nAll other kernel services register with it and provide their introspection data items.";
	}
}
